import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.video.*; 
import static javax.swing.JOptionPane.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class laserTargetApp_6_9_18 extends PApplet {

//Laser Target App
//Kevin Darrah
//http://www.kdcircuits.com


//Libraries 
//you have to go and install this one
//needed by the message box for selecting port

Capture video;//the camera feed "video"

//variables that can be customized
int numberOfBlobs = 10;// number of blobs to have on the screen
int delayBetweenBlobs = 1000;// in milliseconds
int brightnessDiffThreshold = 60; //sensitivity of detection
int blobSizeThreshold = 100; //number of Pixels needed to have changed in order to run the blob detection function

int numPixels;//number pixels scanned each frame

PImage prescanImage;//this is the "old" frame saved
PImage changedImage;//modified frame from old vs new, used to detect the blob

// These are the coordiantes of the blobs
int[] brightSpotX = new int[numberOfBlobs];
int[] brightSpotY = new int[numberOfBlobs];

long startTime_shot = 0;//this is the timer variable for the delay between blobs
boolean firstFrame = true;//just so we can wipe out the changes from the first frame

public void setup() {

  //screen size

  //setup the camera
  cameraInit();

  prescanImage = createImage(640, 480, RGB);
  changedImage = createImage(640, 480, RGB);
  numPixels = 640*480;


}

public void draw() {
  if (video.available()) {//only if a new frame is available
    checkBlob();
    drawScreen();
  }
}
public void checkBlob() {

  //if we see a big blob and there was enough time, since the last blob
  if (pixelsChanged()>blobSizeThreshold && millis()-startTime_shot >delayBetweenBlobs) {
    startTime_shot=millis();
    findBlob();//go find the blob!
  }

  //make a copy of the frame, so we can use it the next time around to check for changes
  prescanImage.copy(video, 0, 0, video.width, video.height, 0, 0, prescanImage.width, prescanImage.height);
  prescanImage.updatePixels();
}




public int pixelsChanged() {
  video.read();
  video.loadPixels();//existing frame
  prescanImage.loadPixels();//last frame

  float pixelBrightness; 
  float oldPixelBrightness;
  int maxPixelsChanged = 0;//keep track of changed pixels, so dust won't trigger blob detection

  //sweep through entire frame comparing existing frame vs previous frame
  for (int y = 0; y< video.height; y++) {
    for (int x = 0; x< video.width; x++) {
      pixelBrightness = brightness(video.pixels[x+y*video.width]);
      oldPixelBrightness = brightness(prescanImage.pixels[x+y*video.width]);
      if (abs(pixelBrightness-oldPixelBrightness)>brightnessDiffThreshold) {// this is the threshold to detect a difference
        changedImage.pixels[x+y*video.width] = color(255, 0, 0);//set changed pixels red
        maxPixelsChanged++;
      } 
       else//if no change, then set to the frame, will look like raw video feed
        changedImage.pixels[x+y*video.width] = video.pixels[x+y*video.width];
    }
  }
  changedImage.updatePixels();//update the pixels, since we changed them

  return maxPixelsChanged;//return the number of pixels changed
}




//here's the magic function to find the blob centers
public void findBlob() {
  int newXSpot= -10, newYSpot= -10;
  int furthestDistance = 0;
  int distanceTracker = 0;
  int xPixelForw=0;
  int xPixelBack=0;
  int yPixelUp=0;
  int yPixelDow=0;
  int spotColor = color(255, 0, 0);

  //scan through the changed image for the 'red' spots
  /* the idea here is to find, first the red pixel that is furthest from the non-red edges, that will give us a good
   *  starting point for the center, then second, we try to find that point that has equal distances away from non-red edges
   */
  for (int y = 0; y< video.height; y++) {
    for (int x = 0; x< video.width; x++) {

      //sweep through looking for red pixels, since this is where our blob lives
      if (color(changedImage.pixels[x+y*video.width])==spotColor) {
        //now we need to walk up and down/left right to find max pixel count away from non red
        for (int i=1; i<(width-x-1); i++) {//let's go forward on x
          if (color(changedImage.pixels[x+i+y*video.width])==spotColor)
            xPixelForw++;
          else
            i=width-x;
        }
        for (int i=1; i<x-1; i++) {//let's go backward on x
          if (color(changedImage.pixels[x-i+y*video.width])==spotColor)
            xPixelBack++;
          else
            i=x;
        }          
        for (int i=1; i<y-1; i++) {//let's go up on y
          if (color(changedImage.pixels[x+(y-i)*video.width])==spotColor)
            yPixelUp++;
          else
            i=y;
        }
        for (int i=1; i<height-y-1; i++) {//let's go down on y
          if (color(changedImage.pixels[x+(y+i)*video.width])==spotColor)
            yPixelDow++;
          else
            i=height-y;
        }
        distanceTracker = xPixelForw+xPixelBack+yPixelUp+yPixelDow;//adds up all of those distances from the non-red edges

        if (distanceTracker>furthestDistance) {//checks for biggest blob
          if (abs(xPixelForw-xPixelBack) < 10 && abs(yPixelUp-yPixelDow) < 10) {//checks for center here, with near-equal distances
            furthestDistance = distanceTracker;//log this as a contender for the big blob
            //set the x,y locations at the 0 position
            newXSpot=x;
            newYSpot=y;
          } else {//reset
            xPixelForw=0;
            xPixelBack=0;
            yPixelUp=0;
            yPixelDow=0;
          }
        } else {//reset
          xPixelForw=0;
          xPixelBack=0;
          yPixelUp=0;
          yPixelDow=0;
        }
      }
    }
  }

  //found a blob, but need to shift the 9 previous over in the array and roll off the last one
  for (int i=numberOfBlobs-1; i>0; i--) {
    brightSpotX[i] = brightSpotX[i-1];
    brightSpotY[i] = brightSpotY[i-1];
  }
  brightSpotX[0] = newXSpot;
  brightSpotY[0] = newYSpot;
}
public void cameraInit() {
  String[] cameras = Capture.list();//create array of available capture devices
  String CameraList="";//will be used for the input box

  if (cameras.length == 0) {
    println("There are no cameras available for capture.");
    exit();
  } else {
    println("Available cameras:");
    for (int i = 0; i < cameras.length; i++) {
      if (cameras[i].contains("size=640x480") && cameras[i].contains("fps=30")) {//only cameras we care about
        print(i);
        print(" ");
        CameraList = CameraList + i + "=" + cameras[i] + "\n";//giant array we post in the box
        println(cameras[i]);
      }
    }

    String CameraSelection="";//the integer we receive back from the box
    CameraSelection = showInputDialog("Please Select Camera (0,1,2,3..):\n" + CameraList);
    if (CameraSelection == null) exit();
    if (CameraSelection.isEmpty()) exit();   
    if (PApplet.parseInt(CameraSelection) >= cameras.length || PApplet.parseInt(CameraSelection)<0) exit();
    video = new Capture(this, cameras[PApplet.parseInt(CameraSelection)]);
  }

  video.start();
}
public void drawScreen() {
  
  if (firstFrame) {
    firstFrame=false;
    for (int i=0; i<numberOfBlobs; i++) {//throw all of the coordinates for the blobs off-screen
      brightSpotX[i]=-10;
      brightSpotY[i]=-10;
    }
  }

  set(0, 0, changedImage);//set the screen to the changed image, so we can see the blobs
  
  noStroke();
  fill(255, 0, 0, 127);//draw the old blobs first
  for (int i=1; i<numberOfBlobs; i++) {
    ellipse(brightSpotX[i], brightSpotY[i], 20, 20);
  }
  fill(0, 255, 0);
  ellipse(brightSpotX[0], brightSpotY[0], 20, 20);
}
public void mousePressed() {//reset everything

  for (int i=0; i<10; i++) {
    brightSpotX[i]=-10;
    brightSpotY[i]=-10;
  }
}
  public void settings() {  size(640, 480); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "laserTargetApp_6_9_18" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
